import 'package:meta/meta.dart';

import 'constants.dart';

@immutable
class Altitude {
  final double _metres;
  const Altitude.fromMetres(num metres) : _metres = metres * 1.0;
  const Altitude.fromFt(num feet) : _metres = feet * mPerFt;
  const Altitude.fromFL(num flightLvl) : _metres = flightLvl * mPerFL;
  const Altitude.nan() : _metres = double.nan;
  const Altitude.infinity() : _metres = double.infinity;
  const Altitude.zero() : _metres = 0.0;

  /// Returns a [int] hash code for a numerical value of [Altitude] instance.
  @override
  int get hashCode => _metres.hashCode;

  /// Returns [Altitude]'s value in metres
  double get m => _metres;

  /// Returns [Altitude]'s value in feet
  double get ft => _metres / mPerFt;

  /// Returns [Altitude]'s value in FL (Flight Levels)
  double get FL => (_metres / mPerFL + 0.5).truncateToDouble();

  /// Whether the [Altitude] has the 0.0 value.
  bool get isZero => _metres == 0.0;

  /// Whether the [Altitude] has the Not-a-Number value.
  bool get isNaN => _metres.isNaN;

  /// Whether the [Altitude] has positive infinity or negative infinity value.
  bool get isInfinite => _metres.isInfinite;

  /// Whether if the [Altitude] is negative.
  /// Negative values are those less than zero, and the `-0.0`.
  bool get isNegative => _metres.isNegative;

  /// Returns -1.0 if the [Altitude] is less than zero,
  /// +1.0 if the [Altitude] is greater than zero,
  /// and the [Altitude] itself if it is -0.0, 0.0 or NaN.
  double get sign => _metres.sign;

  /// String representation of the instance of [Altitude]
  @override
  String toString() => 'Altitude: $_metres metres';

  @override
  bool operator ==(Object other) =>
      other is Altitude && _metres == other._metres;
  bool operator >(Altitude other) => _metres > other._metres;
  bool operator >=(Altitude other) => _metres >= other._metres;
  bool operator <(Altitude other) => _metres < other._metres;
  bool operator <=(Altitude other) => _metres <= other._metres;
  Altitude operator +(Altitude other) =>
      Altitude.fromMetres(_metres + other._metres);
  Altitude operator -(Altitude other) =>
      Altitude.fromMetres(_metres - other._metres);
  Altitude operator *(num other) => Altitude.fromMetres(_metres * other);
  Altitude operator /(num other) => Altitude.fromMetres(_metres / other);
}
