import 'package:meta/meta.dart';

import 'constants.dart';

@immutable
class Azimuth {
  final double _degrees;
  const Azimuth.fromDeg(num degrees) : _degrees = (degrees + 360.0) % 360.0;
  const Azimuth.fromRad(num radians)
      : _degrees = (radians * radToDeg + 360.0) % 360.0;
  const Azimuth.nan() : _degrees = double.nan;
  const Azimuth.infinity() : _degrees = double.infinity;
  const Azimuth.zero() : _degrees = 0.0;

  /// Returns a [int] hash code for a numerical value of [Azimuth] instance.
  @override
  int get hashCode => _degrees.hashCode;

  /// Returns [Azimuth]'s value in degrees
  double get deg => _degrees;

  /// Returns [Azimuth]'s value in radians
  double get rad => _degrees * degToRad;

  /// Whether the [Azimuth] has the 0.0 value.
  bool get isZero => _degrees == 0.0;

  /// Whether the [Azimuth] has the Not-a-Number value.
  bool get isNaN => _degrees.isNaN;

  /// Whether the [Azimuth] has positive infinity or negative infinity value.
  bool get isInfinite => _degrees.isInfinite;

  /// Whether if the [Azimuth] is negative.
  /// Negative values are those less than zero, and the `-0.0`.
  bool get isNegative => _degrees.isNegative;

  /// Returns -1.0 if the [Azimuth] is less than zero,
  /// +1.0 if the [Azimuth] is greater than zero,
  /// and the [Azimuth] itself if it is -0.0, 0.0 or NaN.
  double get sign => _degrees.sign;

  /// String representation of the value of [Azimuth]
  String get toStringDeg => '${_degrees.toStringAsFixed(1)}°';

  /// String representation of the instance of [Azimuth]
  String toString() => 'Azimuth: ${_degrees.toStringAsFixed(1)}°';

  @override
  bool operator ==(Object other) =>
      other is Azimuth && _degrees == other._degrees;
  bool operator >(Azimuth other) => _degrees > other._degrees;
  bool operator >=(Azimuth other) => _degrees >= other._degrees;
  bool operator <(Azimuth other) => _degrees < other._degrees;
  bool operator <=(Azimuth other) => _degrees <= other._degrees;
  Azimuth operator +(Azimuth other) =>
      Azimuth.fromDeg(_degrees + other._degrees);
  Azimuth operator -(Azimuth other) =>
      Azimuth.fromDeg(_degrees - other._degrees);
  Azimuth operator *(num other) => Azimuth.fromDeg(_degrees * other);
  Azimuth operator /(num other) => Azimuth.fromDeg(_degrees / other);
}
