import 'package:meta/meta.dart';

import 'constants.dart';

@immutable
class Speed {
  final double _kmh;
  const Speed.fromKmh(num kmh) : _kmh = kmh * 1.0;
  const Speed.fromKt(num knots) : _kmh = knots * kmhPerKt;
  const Speed.fromMph(num mph) : _kmh = mph * kmhPerMph;
  const Speed.fromMs(num ms) : _kmh = ms * kmhPerMs;
  const Speed.fromKmMin(num kmMin) : _kmh = kmMin * kmhPerKmMin;
  const Speed.fromFtMin(num ftMin)
      : _kmh = ftMin * mPerFt * mPerKm * kmhPerKmMin;
  const Speed.nan() : _kmh = double.nan;
  const Speed.infinity() : _kmh = double.infinity;
  const Speed.zero() : _kmh = 0.0;

  /// Returns a [int] hash code for a numerical value of [Speed] instance.
  @override
  int get hashCode => _kmh.hashCode;

  /// Returns [Speed]'s value in kilometres per hour
  double get kmh => _kmh;

  /// Returns [Speed]'s value in knots (nautical mile per hour)
  double get kt => _kmh / kmhPerKt;

  /// Returns [Speed]'s value in (statute) mile per hour
  double get mph => _kmh / kmhPerMph;

  /// Returns [Speed]'s value in metres per second
  double get ms => _kmh / kmhPerMs;

  /// Returns [Speed]'s value in kilometres per minute
  double get kmMin => _kmh / kmhPerKmMin;

  /// Returns [Speed]'s value in kilometres per hour
  double get ftMin => _kmh / mPerKm / mPerFt / kmhPerKmMin;

  /// Whether the [Speed] has the 0.0 value.
  bool get isZero => _kmh == 0.0;

  /// Whether the [Speed] has the Not-a-Number value.
  bool get isNaN => _kmh.isNaN;

  /// Whether the [Speed] has positive infinity or negative infinity value.
  bool get isInfinite => _kmh.isInfinite;

  /// Whether if the [Speed] is negative.
  /// Negative values are those less than zero, and the `-0.0`.
  bool get isNegative => _kmh.isNegative;

  /// Returns -1.0 if the [Speed] is less than zero,
  /// +1.0 if the [Speed] is greater than zero,
  /// and the [Speed] itself if it is -0.0, 0.0 or NaN.
  double get sign => _kmh.sign;

  /// String representation of the instance of [Speed]
  @override
  String toString() => 'Speed: $_kmh kmh';

  @override
  bool operator ==(Object other) => other is Speed && _kmh == other._kmh;
  bool operator >(Speed other) => _kmh > other._kmh;
  bool operator >=(Speed other) => _kmh >= other._kmh;
  bool operator <(Speed other) => _kmh < other._kmh;
  bool operator <=(Speed other) => _kmh <= other._kmh;
  Speed operator +(Speed other) => Speed.fromKmh(_kmh + other._kmh);
  Speed operator -(Speed other) => Speed.fromKmh(_kmh - other._kmh);
  Speed operator *(num other) => Speed.fromKmh(_kmh * other);
  Speed operator /(num other) => Speed.fromKmh(_kmh / other);
}
