import 'package:meta/meta.dart';

import 'constants.dart';

@immutable
class Time {
  final double _seconds;
  const Time.fromSeconds(num seconds) : _seconds = seconds * 1.0;
  const Time.fromMinutes(num minutes) : _seconds = minutes * secPerMin;
  const Time.fromHours(num hours) : _seconds = hours * secPerHour;
  const Time.fromDays(num days) : _seconds = days * secPerDay;
  const Time.fromWeeks(num weeks) : _seconds = weeks * daysPerWeek * secPerDay;
  const Time.nan() : _seconds = double.nan;
  const Time.infinity() : _seconds = double.infinity;
  const Time.zero() : _seconds = 0.0;

  /// Returns a [int] hash code for a numerical value of [Time] instance.
  @override
  int get hashCode => _seconds.hashCode;

  /// Returns [Time]'s value in seconds
  double get sec => _seconds;

  /// Returns [Time]'s value in minutes
  double get min => _seconds / 60.0;

  /// Returns [Time]'s value in hours
  double get hrs => _seconds / 60.0 / 60.0;

  /// Returns [Time]'s value in days
  double get days => _seconds / 60.0 / 60.0 / 24.0;

  /// Returns [Time]'s value in weeks
  double get weeks => _seconds / 60.0 / 60.0 / 24.0 / 7.0;

  /// Whether the [Time] has the 0.0 value.
  bool get isZero => _seconds == 0.0;

  /// Whether the [Time] has the Not-a-Number value.
  bool get isNaN => _seconds.isNaN;

  /// Whether the [Time] has positive infinity or negative infinity value.
  bool get isInfinite => _seconds.isInfinite;

  /// Whether if the [Time] is negative.
  /// Negative values are those less than zero, and the `-0.0`.
  bool get isNegative => _seconds.isNegative;

  /// Returns -1.0 if the [Time] is less than zero,
  /// +1.0 if the [Time] is greater than zero,
  /// and the [Time] itself if it is -0.0, 0.0 or NaN.
  double get sign => _seconds.sign;

  /// String representation of the instance of [Time]
  @override
  String toString() => 'Time: $_seconds seconds';

  @override
  bool operator ==(Object other) => other is Time && _seconds == other._seconds;
  bool operator >(Time other) => _seconds > other._seconds;
  bool operator >=(Time other) => _seconds >= other._seconds;
  bool operator <(Time other) => _seconds < other._seconds;
  bool operator <=(Time other) => _seconds <= other._seconds;
  Time operator +(Time other) => Time.fromSeconds(_seconds + other._seconds);
  Time operator -(Time other) => Time.fromSeconds(_seconds - other._seconds);
  Time operator *(num other) => Time.fromSeconds(_seconds * other.toDouble());
  Time operator /(num other) => Time.fromSeconds(_seconds / other.toDouble());
}
